<?php

namespace Ecommerce\Controllers;

use Ecommerce\Util\RepositoryInstance;
use Teknisa\Libs\Service\Menu;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;

class LoginController
{
    const CHANGE_PASSWORD_ROUTE = "product#AlterarSenha";
    
    /** @var \Services\Login $service */
    protected $service;
    protected $utilities;
    protected $email;
    protected $connection;
    protected $environment;
    protected $session;
    protected $orgId;

    /**
     * @param \Services\Login $loginService
     */
    public function __construct($service, $utilities, $email, $environment, $sessionManager, $orgId) {
        $this->service = $service;
        $this->utilities = $utilities;
        $this->email = $email;
        $this->environment = $environment;
        $this->session = $sessionManager;
        $this->orgId = $orgId;
    }
    
    public function logout(Request\Row $request, Response $response) 
    {
        $token = $this->environment->getLoginTokenInSession();
        
        $this->session->start();
        
        $dataSet = array (
            "requestType" => "FilterData",
            "filter" => array (
                array (
                    'name' => 'TOKEN',
                    'operator' => '=',
                    'value' => $token
                )
            )
        );
        try {
            $this->utilities->callWebService($token, '/lm_logout', $dataSet, 'logout', $this->service->loginClientSecret, $this->service->loginUrl, 'Yes');
        } catch(\Exception $e) {}

        $this->service->getOAuthCustom()->deleteToken($token);
        $this->session->destroy();
    }
    
    /**
     * @param Request\Row $request
     * @param Response $response
     */
    public function login(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $return = array();
           
            $userData = $this->service->getUserData($row);
            if (!empty($userData)) {
                $this->service->getLibConnectionCustom()->updateVpdConnection($userData[0]['NRORG']);
                $loginResponse = $this->getLoginResponse($row, $userData);
                
                $loginResponse["TOKEN"] = $this->generateToken($loginResponse);
                
                if (sizeof($userData) > 1) {
                    $return = $this->createReturnArray(false, "MULTIPLE_FORN");
                } else {
                    $return = $this->createReturnArray(false, "SINGLE_FORN");
                }
                $this->createAdditionalData($return, $row, $userData, $loginResponse);
                $this->saveDataOnSession($return);
            } else {
                $return = $this->createReturnArray(true, "ERR_LOGIN_DATA_INCORRECT");
            }
        } catch(CustomException $e) {
            $return = $this->createReturnArray(true, "ERR_DB_QUERY_EXEC");
            $return['ORIGINAL'] = $error->getMessage();
        } catch (\Exception $error){
            $return = $this->createReturnArray(true, "ERR_UNEXPECTED_ERROR");
            $return['ORIGINAL'] = $error->getMessage();
        } finally {
            $response->addDataSet(new DataSet ('request', $return));
        }
    }
    
    private function getLoginResponse($row, $userData)
    {
        $organizationId = $this->orgId && $this->orgId !== "SAAS" ? $this->orgId : $userData[0]['NRORG'];
        $filter = $this->getLoginParams($organizationId, $row["PRODUCT_ID"]);
        return $this->utilities->callWebService(null, '/productLicense', $filter, 'userData', $this->service->loginClientSecret, $this->service->loginUrl);
    }
    
    private function createAdditionalData(&$return, $row, $userData, $loginResponse)
    {
        $userInfo = $this->createUserInfoToEncode($userData, $row, $loginResponse);
        $encryptedSession = $this->utilities->zeedhiEncrypt(json_encode($userInfo));

        $return["USER_ID"] = $userData[0]['CDOPERADOR'];
        $return["USER_NAME"] = $userData[0]['NMOPERADOR'];
        $return["USER_EMAIL"] = $row["USER_EMAIL"];
        
        $return["REDIRECT_URL"] = $loginResponse["REDIRECT_URL"];
        $return["TOKEN"] = $loginResponse["TOKEN"];
        $return["USER"] = $encryptedSession;
        $return["LOGGED"] = true;
        $return["ORGANIZATION_ID"] = $userData[0]['NRORG'];
        $return["VISIBLE_ELEMENTS"] = $loginResponse["VISIBLE_ELEMENTS"];
        $return["ACCESS_PROFILE"][0] = "11";
        $return["ACL_ADMINISTRATOR"] = false;
        $return["SESSION_EXPIRED"] = $this->environment->getSessionExpired();
    }
    
    protected function getLoginParams($organizationId, $productId) 
    {
        $menus = $this->service->getLibMenuService()->buildMenu(true, array(Menu::ALL_ELEMENTS_VISIBLE), null, null, null, null);
        $elementIds = array();
        $this->getElementIds($menus, $elementIds);
        return array (
            "requestType" => "FilterData",
            "filter" => array( array(
                'name' => 'ORGANIZATION_ID', 'operator' => '=', 'value' => $organizationId
            ), array (
                'name' => 'PRODUCT_ID',      'operator' => '=', 'value' => $productId
            ), array (
                'name' => 'ELEMENT_IDS',     'operator' => '=', 'value' => $elementIds
            ) )
        );
    }
    
    protected function getElementIds($menus, &$elementIds) 
    {
        $menus = isset($menus['MENUS']) ? $menus['MENUS'] : $menus;
        
        foreach ($menus as $menu) {
            if (isset($menu['id'])) {
                array_push($elementIds, $menu['id']);
            }
            
            if (isset($menu['menus'])) {
                $this->getElementIds($menu['menus'], $elementIds);
            }
        }
    }
    
    private function saveDataOnSession($data)
    {
        $this->environment->setLoginTokenInSession($data["TOKEN"]);
        $this->environment->setCurrentUserId($data["USER"]);
        $this->environment->getCurrentUserName($data["USER_NAME"]);
        $this->environment->setCdOperador($data["USER_ID"]);
        $this->environment->setNrOrg($data["ORGANIZATION_ID"]);
        $this->environment->setSessionExpired(false);
        $this->service->getSession()->set('NRORG', $data["ORGANIZATION_ID"]);
    }
    
    private function createUserInfoToEncode($userData, $row, $loginResponse)
    {
        return array(
            "USER_ID" => $userData[0]['CDOPERADOR'],
            "USER_EMAIL" => $row["USER_EMAIL"],
            "TOKEN" => $loginResponse["TOKEN"],
            "REDIRECT_URL" => $loginResponse["REDIRECT_URL"],
            "VISIBLE_ELEMENTS" => $loginResponse["VISIBLE_ELEMENTS"],
            "EXPIRED_ELEMENTS" => isset($loginResponse['EXPIRED_ELEMENTS']) ? $loginResponse['EXPIRED_ELEMENTS'] : [],
            "REPOSITORIES_INFO" => $loginResponse['REPOSITORIES_INFO'],
            "USER_DATA" => $userData,
            "FILE_SERVER_API_KEY" => isset($loginResponse["FILE_SERVER_API_KEY"]) ? $loginResponse["FILE_SERVER_API_KEY"] : null,
        );
    }
    
    public function checkActiveSession(Request\Row $request, Response $response)
    {
        try {
            $return = array();
            $row = $request->getRow();
            
            $userInfo = @json_decode($this->utilities->zeedhiDecrypt($row["USER"]), true);

            if (!empty($userInfo)) {
                $return = $this->createReturnArray(false, "SUC_REQ");
                $this->createAdditionalData($return, $userInfo, $userInfo["USER_DATA"], $userInfo);
            }
        } catch(CustomException $e) {
            $return = $this->createReturnArray(true, "ERR_DB_QUERY_EXEC");
        } finally {
            $response->addDataSet(new DataSet ('request', $return));
        }
    }
    
    private function generateToken($loginResponse)
    {
        $token = $loginResponse["TOKEN"] ?? null;
        
        return $this->service->getAuthImpl()->grantAccessToken(
            $this->service->libClientId,
            $this->service->libClientSecret,
            array('sessionId' => $this->service->getSession()->getId()),
            $token
        );
    }
    
    /*****************************************/
    /***** PASSWORD REDEFINITION PROCESS *****/
    /*****************************************/
    
    /**
     * @param Request\Filter $request
     * @param Response $response
     */
    public function requestNewPassword(Request\Row $request, Response $response) 
    {
        try {
            $this->service->beginTransaction();
            
            $row = $request->getRow();
            
            $user = $this->service->checkUserExist($row["EMAIL"]);
            if (isset($user[0]["CDFORNECED"])) {
                $token = $this->utilities->generateRandomToken(20);
                
                $this->service->inactivateOtherRequests($row, $token);
                $this->service->insertLogRequestPassword($row, $token);
                $this->sendMailConfirmResetPassword($user, $token);

                $this->service->commitTransaction();
                $return = $this->createReturnArray(false, "SUC_REQ_PASS_REDEF");
            } else {
                $this->service->rollbackTransaction();
                $return = $this->createReturnArray(true, "ERR_EMAIL_NOT_FOUND");
            }
        } catch(\Exception $e) {
            $this->service->rollbackTransaction();
            if($e->getMessage() == "Central de compras não encontrada.") {
                $return = $this->createReturnArray(true, "UNIT_NOT_FOUND");
            } else {
                $return = $this->createReturnArray(true, "ERR_DB_QUERY_EXEC");
            }
        } finally {
            $response->addDataSet(new DataSet('request', $return));
        }
    }
    
    private function sendMailConfirmResetPassword($user, $token)
    {
        $link = $this->generateResetPasswordLink($user, $token);
        $email = $this->createMessageResetPassword($user, $link);
        $unit = $this->email->getFilialParasupri();
        $this->email->setUnitId($unit);
        $this->email->useCustomSmtp();
        $this->email->send($email["SUBJECT"], $email["CONTENT"], $user[0]["DSLOGINECO"]);
    }
    
    private function generateResetPasswordLink($user, $token)
    {
        $host = $_SERVER["HTTP_REFERER"];
        $email = $user[0]["DSLOGINECO"];
        
        $link = $host . "#/" . self::CHANGE_PASSWORD_ROUTE . "?token=" . $token . "&email=" . $email;
        
        return $link;
    }
    
    private function createMessageResetPassword($user, $link)
    {
        $file = array("name" => "reset-password", "extension" => ".html");
        $replace = array(
            "%USER%" => $this->utilities->capitalizeWords($user[0]["NMUSUARIOECO"]), 
            "%LINK%" => $link
        );

        $email = array();
        
        $email["SUBJECT"] = "Alteração de Senha";
        
        $email["CONTENT"] = $this->email->getTemplate($file, $replace);
        
        return $email;
    }
    
    private function createReturnArray($error, $code = "")
    {
        return array(
            "ERROR" => $error,
            "MESSAGE" => $code
        );
    }
    
    public function validateLinkStillActive(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();
            
            $request = $this->service->getRequestStatus($row);
            $checkRequest = $this->checkRequestIsValid($request);
            
            if ($checkRequest) {
                switch ($request[0]["IDATIVO"]) {
                    case 'S':
                        $return = $this->createReturnArray(false);
                    break;
                    
                    case 'N':
                        if (isset($request[0]["DTREDEFINICAO"]) && $request[0]["DTREDEFINICAO"] != "") {
                            $return = $this->createReturnArray(true, "ERR_LINK_ALR_USED");
                        } else {
                            $return = $this->createReturnArray(true, "ERR_ANOTHER_LINK_GEN");
                        }
                    break;
                }
            } else {
                $return = $this->createReturnArray(true, "ERR_LINK_INC_PARAMS");
            }
        } catch (\Exception $e) {
            $return = $this->createReturnArray(true, "ERR_DB_QUERY_EXEC");
        } finally {
            $response->addDataSet(new DataSet('request', $return));
        }
    }
    
    public function redefinePassword(Request\Row $request, Response $response)
    {
        try {
            $this->service->beginTransaction();
            $row = $request->getRow();
            
            $request = $this->service->getRequestStatus($row);
            
            $checkRequest = $this->checkRequestIsValid($request);
            
            if ($checkRequest) {
                $checkPassword = $this->validateNewPassword($row["PASSWORD"], $row["PASSWORD_CONFIRMATION"]);
                
                if($checkPassword) {
                    
                    $this->service->updateLogRequestPassword($row);
                    $this->service->redefinePassword($row);
                    $this->service->commitTransaction();
                    
                    $return = $this->createReturnArray(false, "SUC_REDEF_PASS");
                } else {
                    $this->service->rollbackTransaction();
                    $return = $this->createReturnArray(true, "ERR_PASSWORD_REQ_MISS");
                }
            } else {
                $this->service->rollbackTransaction();
                $return = $this->createReturnArray(true, "ERR_LINK_INC_PARAMS");
            }
        } catch (\Exception $e) {
            $this->service->rollbackTransaction();
            $return = $this->createReturnArray(true, "ERR_DB_QUERY_EXEC");
        } finally {
            $response->addDataSet(new DataSet('request', $return));
        }
    }
    
    private function checkRequestIsValid($request) {
        return isset($request[0]["IDATIVO"]) ? true : false;
    }
    
    /**
     * Validate new password
     * 
     * Receives the new password and it's confirmation, then validate if they are equal and
     * if the new password is according to the validation:
     * 
     * At least 2 uppercase letters;
     * At least 3 lowercase letters;
     * At least 2 numbers;
     * At least 1 special character;
     * String must be between 8 and 12 characters
     *
     * @param string $newPassword String containing new password.
     * @param string $newPasswordConfirmation String containing new password confirmation.
     *
     * @return bool
     * @throws Exception
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function validateNewPassword($newPassword, $newPasswordConfirmation)
    {
        if ($newPassword === $newPasswordConfirmation) {
            $uppercase = preg_match_all('@[A-Z]@', $newPassword);
            $lowercase = preg_match_all('@[a-z]@', $newPassword);
            $number = preg_match_all('@[0-9]@', $newPassword);
            $specialChars = preg_match_all('@[^\w]@', $newPassword);
            
            if ($lowercase >= 3 && $uppercase >= 2 && $number >= 2 && $specialChars >= 1 && (strlen($newPassword) >= 8 && strlen($newPassword) <= 12)) {
                return true;
            } else {
                throw new \Exception("A nova senha deve possuir no mínimo: 3 caracteres minusculos, 2 caracteres maiúsculos, 1 caractere especial, 1 número e entre 8 a 12 caracteres.", 400);
            }
        } else {
            throw new \Exception("A nova senha é diferente da confirmação", 400);
        }
    }
    
    /**
     * @param Request\Filter $request
     * @param Response $response
     */
    public function getProductLanguages(Request\Filter $request, Response $response) 
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            
            $productId  = $this->utilities->getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID');
            
            $languages  = $this->service->getProductLanguages($productId);
            
            $response->addDataSet(new DataSet('languages', $languages));
        } catch(CustomException $e) {
            $response->setError($this->utilities->getErrorObjectFromException($e));
        }
    }
    
    /**
     * @param Request\Filter $request
     * @param Response $response
     */
    public function testLicenseConnection(Request\Filter $request, Response $response) 
    {
        try {
            $result = $this->service->testLicenseConnection();
            
            $response->addDataSet(new DataSet('testLicenseConnection', $result));
        } catch(CustomException $e) {
            $response->setError($this->utilities->getErrorObjectFromException($e));
        }
    }
    
    public function validateUser(Request\Row $request, Response $response){
        try {
            
            $return = array();
            $requestRow     = $request->getRow();
            $cdoperador     = $requestRow['USER']     ?? '';
            $password       = $requestRow['PASSWORD'] ?? '';
            $valid         = $this->service->validateUser($cdoperador, $password);
            $return = array(
                'IS_VALID' => $valid    
            );
            
        } catch (\Exception $error){
            $return = $this->createReturnArray(true, "ERR_UNEXPECTED_ERROR");
        } finally {
            $response->addDataSet(new DataSet ('userValidation', $return));
        }
        
    }
    
    public function redefinePasswordForne(Request\Row $request, Response $response)
    {
        try {
            $this->service->beginTransaction();
            $row = $request->getRow();
    
            $newPassword = $row["PASSWORD"];
            
            if ($row["PASSWORD"] === $row["PASSWORD_CONFIRMATION"]) {
                $uppercase = preg_match_all('@[A-Z]@', $newPassword);
                $lowercase = preg_match_all('@[a-z]@', $newPassword);
                $number = preg_match_all('@[0-9]@', $newPassword);
                $specialChars = preg_match_all('@[^\w]@', $newPassword);
                if ($lowercase >= 3 && $uppercase >= 2 && $number >= 2 && $specialChars >= 1 && (strlen($newPassword) >= 8 && strlen($newPassword) <= 12)) {
                    $this->service->redefinePassword($row);
                    $this->service->commitTransaction();
                    $return = $this->createReturnArray(false, "SUC_REDEF_PASS");
                } else {
                    $return = $this->createReturnArray(true, "A nova senha deve possuir no mínimo: 3 caracteres minusculos, 2 caracteres maiúsculos, 1 caractere especial, 1 número e entre 8 a 12 caracteres.");
                }
            } else {
                $return = $this->createReturnArray(true, "A nova senha é diferente da confirmação");
            }
        } catch (\Exception $e) {
            $this->service->rollbackTransaction();
            $return = $this->createReturnArray(true, "ERR_DB_QUERY_EXEC");
        } finally {
            $response->addDataSet(new DataSet('request', $return));
        }
    }
    
    public function verificaLogin(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $return = array();
           
            $userData =$this->service->getUserData($row);
            if (!empty($userData)) {

            if (sizeof($userData) > 1) {
                    $return = $this->createReturnArray(false, "MULTIPLE_FORN");
                } else {
                    $return = $this->createReturnArray(false, "SINGLE_FORN");
                }

            } else {
                $return = $this->createReturnArray(true, "ERR_LOGIN_DATA_INCORRECT");
            }
        }catch (\Exception $error){
            $return = $this->createReturnArray(true, "ERR_UNEXPECTED_ERROR");
        } finally {
            $response->addDataSet(new DataSet ('request', $return));
        }
    }
    
    
}