<?php
namespace Ecommerce\Controllers;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;

class Sup09400 extends \Zeedhi\Framework\Controller\Report
{
    protected $birtStrategy;
    protected $environment;
    protected $reportService;
    protected $QR2Strategy;
    protected $serviceEmail;
    protected $fileServerURL;
    protected $connection;
    
    public function __construct($reportService, $birtStrategy, $environment, $QR2Strategy, $serviceEmail, $fileServerURL, $connection) {
        $this->reportService     = $reportService;
        $this->birtStrategy      = $birtStrategy;
        $this->environment       = $environment;
        $this->QR2Strategy       = $QR2Strategy;
        $this->serviceEmail = $serviceEmail;
        $this->fileServerURL     = $fileServerURL;
        $this->connection   = $connection;
    }
    
    /** @var array */
    protected $reportMapping = array (
        'SUP09400' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        "CDFORNECED"     => "P_CDFORNECED",
                        "CDFILIAL"       => "P_CDFILIAL",
                        "NRPEDIDOCOL"    => "P_NRPEDIDOCOL",
                        "NRORG"          => "P_NRORG"
                    )
            ),
            'SUP09402' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        "CDFORNECED"     => "P_CDFORNECED",
                        "CDFILIAL"       => "P_CDFILIAL",
                        "NRPEDIDOCOL"    => "P_NRPEDIDOCOL",
                        "CDPICTPROD"     => "P_CDPICTPROD",
                        "NRORG"          => "P_NRORG"
                    )
            ),
            'SUP09410' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        "CDFORNECED"     => "P_CDFORNECED",
                        "CDFILIAL"       => "P_CDFILIAL",
                        "NRPEDIDOCOL"    => "P_NRPEDIDOCOL",
                        "NRORG"          => "P_NRORG"
                    )
            )
    );
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $row['NRORG'] = $this->environment->getNrorgTrab();
        $qr2type = isset($row['QR2TYPE']) ? $row['QR2TYPE'] : 'PDF';
        $this->birtStrategy->setReportFormat($qr2type);
        $this->QR2Strategy->setQr2Type($qr2type);
    }
    
    public function afterProcessReport(DTO\Request\Row $request, DTO\Response $response, $report) {
        $row = $request->getRow();
        if($row['__report_name'] == 'SUP09402') {
            try{
                $this->serviceEmail->enviaEmailInconsistencia($row, $report, $response);
            } catch(\Exception $e){
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), 1));
            }
        }
    }
    
    private function logoSrc($response){
        try{
			$sql = "SELECT URLLOGO FROM APPCONFIG WHERE IDAPLICATIVO = 'PORTAL E-COMMERCE' AND NRORG = :P_NRORG";
            $stmt = $this->connection->prepare($sql);
            $stmt->execute(
                array(
                    "P_NRORG" => $this->environment->getNrorgTrab()
                )
            );
            
			$result = $stmt->fetchAll();
			
			if(isset($result[0]) && $result[0]['URLLOGO'] && $result[0]['URLLOGO'] != null && $result[0]['URLLOGO'] != '') {
    			$logoSrc = $this->fileServerURL . $result[0]['URLLOGO'];
    			return $logoSrc;
	        } else {
	            return false;
	        }
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), 1));
        }
    
    }

}